@@ -19,6 +19,18 @@ from sanic.response import html, text
 
 
 class ErrorHandler:
+    """
+    Provide :class:`sanic.app.Sanic` application with a mechanism to handle
+    and process any and all uncaught exceptions in a way the application
+    developer will set fit.
+
+    This error handling framework is built into the core that can be extended
+    by the developers to perform a wide range of tasks from recording the error
+    stats to reporting them to an external service that can be used for
+    realtime alerting system.
+
+    """
+
     handlers = None
     cached_handlers = None
     _missing = object()
@@ -58,9 +70,34 @@ class ErrorHandler:
         )
 
     def add(self, exception, handler):
+        """
+        Add a new exception handler to an already existing handler object.
+
+        :param exception: Type of exception that need to be handled
+        :param handler: Reference to the method that will handle the exception
+
+        :type exception: :class:`sanic.exceptions.SanicException` or
+            :class:`Exception`
+        :type handler: ``function``
+
+        :return: None
+        """
         self.handlers.append((exception, handler))
 
     def lookup(self, exception):
+        """
+        Lookup the existing instance of :class:`ErrorHandler` and fetch the
+        registered handler for a specific type of exception.
+
+        This method leverages a dict lookup to speedup the retrieval process.
+
+        :param exception: Type of exception
+
+        :type exception: :class:`sanic.exceptions.SanicException` or
+            :class:`Exception`
+
+        :return: Registered function if found ``None`` otherwise
+        """
         handler = self.cached_handlers.get(type(exception), self._missing)
         if handler is self._missing:
             for exception_class, handler in self.handlers:
@@ -75,9 +112,15 @@ class ErrorHandler:
         """Fetches and executes an exception handler and returns a response
         object
 
-        :param request: Request
+        :param request: Instance of :class:`sanic.request.Request`
         :param exception: Exception to handle
-        :return: Response object
+
+        :type request: :class:`sanic.request.Request`
+        :type exception: :class:`sanic.exceptions.SanicException` or
+            :class:`Exception`
+
+        :return: Wrap the return value obtained from :func:`default`
+            or registered handler for that type of exception.
         """
         handler = self.lookup(exception)
         response = None
@@ -109,6 +152,20 @@ class ErrorHandler:
         """
 
     def default(self, request, exception):
+        """
+        Provide a default behavior for the objects of :class:`ErrorHandler`.
+        If a developer chooses to extent the :class:`ErrorHandler` they can
+        provide a custom implementation for this method to behave in a way
+        they see fit.
+
+        :param request: Incoming request
+        :param exception: Exception object
+
+        :type request: :class:`sanic.request.Request`
+        :type exception: :class:`sanic.exceptions.SanicException` or
+            :class:`Exception`
+        :return:
+        """
         self.log(format_exc())
         try:
             url = repr(request.url)
@@ -133,7 +190,23 @@ class ErrorHandler:
 
 
 class ContentRangeHandler:
-    """Class responsible for parsing request header"""
+    """
+    A mechanism to parse and process the incoming request headers to
+    extract the content range information.
+
+    :param request: Incoming api request
+    :param stats: Stats related to the content
+
+    :type request: :class:`sanic.request.Request`
+    :type stats: :class:`posix.stat_result`
+
+    :ivar start: Content Range start
+    :ivar end: Content Range end
+    :ivar size: Length of the content
+    :ivar total: Total size identified by the :class:`posix.stat_result`
+        instance
+    :ivar ContentRangeHandler.headers: Content range header ``dict``
+    """
 
     __slots__ = ("start", "end", "size", "total", "headers")
 
