@@ -18,7 +18,7 @@ Parameter = namedtuple("Parameter", ["name", "cast"])
 REGEX_TYPES = {
     "string": (str, r"[^/]+"),
     "int": (int, r"-?\d+"),
-    "number": (float, r"-?[0-9\\.]+"),
+    "number": (float, r"-?(?:\d+(?:\.\d*)?|\.\d+)"),
     "alpha": (str, r"[A-Za-z]+"),
     "path": (str, r"[^/].*?"),
     "uuid": (
@@ -406,6 +406,7 @@ class Router:
         if not self.hosts:
             return self._get(request.path, request.method, "")
         # virtual hosts specified; try to match route to the host header
+
         try:
             return self._get(
                 request.path, request.method, request.headers.get("Host", "")
